import { __awaiter, __decorate } from "tslib";
import { BaseDatasource } from '@genesislcap/foundation-ui';
import { createLogger, LifecycleMixin } from '@genesislcap/foundation-utils';
import { attr, customElement } from '@microsoft/fast-element';
import { extractFields, getFormattedDateGroupBy, getReduced } from './chart-datasource.utils';
const logger = createLogger('chart-datasource');
export const defaultChartFields = 'groupBy value';
export const defaultSeriesChartField = 'series';
const defaultConfigFieldsCount = 2;
const maxConfigFieldsCount = 3;
const groupByConfigFieldIndex = 0;
const valueConfigFieldIndex = 1;
const seriesConfigFieldIndex = 2;
class ConcreteDatasource extends BaseDatasource {
    constructor() {
        super(...arguments);
        this.chartFields = defaultChartFields;
        this.isStack = false;
        this.isGroup = false;
        this.reduced = true;
        this.showFormattedTime = false;
    }
    connectedCallback() {
        const _super = Object.create(null, {
            connectedCallback: { get: () => super.connectedCallback }
        });
        return __awaiter(this, void 0, void 0, function* () {
            _super.connectedCallback.call(this);
            this.chart = this.parentElement;
            this.extractedServerFields = extractFields(this.serverFields);
            this.extractedChartsFields = extractFields(this.chartFields);
            this.prepareSeriesField();
            if (!this.hasValidServerFields || !this.hasValidChartFields) {
                logger.warn('Use valid server and chart fields (e.g server-fields="NAME VALUE" charts-fields="groupBy value"). Chart will not load.');
                return;
            }
            yield this.initDatasource();
            const invalidServerFieldsFound = this.prepareValidServerFieldsMetadata();
            if (invalidServerFieldsFound.length > 0) {
                const allPossibleValidServerFields = this.datasource.fieldMetadata.map((field) => field.name);
                const allInvalidServerFields = invalidServerFieldsFound.join(' ');
                const message = invalidServerFieldsFound.length > 1
                    ? `Server fields '${allInvalidServerFields}' are invalid.`
                    : `Server field '${allInvalidServerFields}' is invalid.`;
                logger.warn(`${message} Chart will not load. Available fields:`, allPossibleValidServerFields);
                return;
            }
            if (this.chart && this.resourceName && this.connect.isConnected) {
                this.fetchGenesisData(false);
            }
        });
    }
    clearData() {
        var _a;
        (_a = this.dataSub) === null || _a === void 0 ? void 0 : _a.unsubscribe();
        this.rowsSyncedWithStream = new Map();
        this.chart.data = [];
    }
    syncComponentData(snapshotData) {
        const rowData = snapshotData !== null && snapshotData !== void 0 ? snapshotData : Array.from(this.rowsSyncedWithStream.values());
        let chartData = [];
        rowData.forEach((row) => {
            const chartRow = {};
            this.extractedServerFields.forEach((fieldName, index) => {
                if (this.withTimestampFormatting && fieldName === this.groupByServerFieldName) {
                    row[fieldName] = getFormattedDateGroupBy(row[fieldName], this.validServerFieldsMetadata.get(fieldName).type, this.showFormattedTime);
                }
                chartRow[this.extractedChartsFields[index]] = row[fieldName];
            });
            chartData.push(chartRow);
        });
        if (this.reduced) {
            chartData = getReduced(chartData, this.extractedChartsFields[groupByConfigFieldIndex], this.extractedChartsFields[valueConfigFieldIndex], this.seriesChartField);
        }
        if (this.chart.type === 'dualaxes') {
            this.chart.data = [chartData, chartData];
        }
        else {
            this.chart.data = chartData;
        }
        this.requiresFullDataRefresh = false;
    }
    handleStreamInserts(insertedRows) {
        insertedRows === null || insertedRows === void 0 ? void 0 : insertedRows.forEach((insertData) => {
            this.rowsSyncedWithStream.set(insertData[this.rowId], insertData);
        });
        this.syncComponentData();
    }
    handleStreamDeletes(deletedRows) {
        deletedRows === null || deletedRows === void 0 ? void 0 : deletedRows.forEach((deleteData) => {
            this.rowsSyncedWithStream.delete(deleteData[this.rowId]);
        });
        this.syncComponentData();
    }
    handleStreamUpdates(updatedRows) {
        updatedRows === null || updatedRows === void 0 ? void 0 : updatedRows.forEach((updateData) => {
            const rowToBeUpdated = this.rowsSyncedWithStream.get(updateData[this.rowId]);
            const updatedRow = Object.assign(Object.assign({}, rowToBeUpdated), updateData);
            this.rowsSyncedWithStream.set(updateData[this.rowId], updatedRow);
        });
        this.syncComponentData();
    }
    prepareSeriesField() {
        if (this.extractedServerFields.length === maxConfigFieldsCount &&
            this.extractedChartsFields.length === defaultConfigFieldsCount) {
            this.extractedChartsFields.push(defaultSeriesChartField);
        }
        if (this.extractedChartsFields.length === maxConfigFieldsCount) {
            this.seriesChartField = this.extractedChartsFields[seriesConfigFieldIndex];
        }
        if (this.seriesChartField) {
            this.chart.config.seriesField = this.seriesChartField;
            switch (this.chart.type) {
                case 'bar':
                case 'column':
                    if (this.isStack && this.isGroup) {
                        logger.warn('Charts can not be stacked and grouped at the same time. Chart will not load properly.');
                    }
                    this.chart.config.isStack = this.isStack;
                    this.chart.config.isGroup = this.isGroup;
                    break;
            }
        }
    }
    prepareValidServerFieldsMetadata() {
        const invalidServerFieldNamesFound = [];
        this.validServerFieldsMetadata = new Map();
        this.extractedServerFields.forEach((fieldName) => {
            const fieldMetadata = this.datasource.fieldMetadata.find((fm) => fm.name === fieldName);
            if (fieldMetadata) {
                this.validServerFieldsMetadata.set(fieldName, fieldMetadata);
            }
            else {
                invalidServerFieldNamesFound.push(fieldName);
            }
        });
        return invalidServerFieldNamesFound;
    }
    get groupByServerFieldName() {
        return this.extractedServerFields[groupByConfigFieldIndex];
    }
    get hasValidServerFields() {
        return this.hasValidFieldsStructure(this.extractedServerFields);
    }
    get hasValidChartFields() {
        return this.hasValidFieldsStructure(this.extractedChartsFields);
    }
    hasValidFieldsStructure(fieldsArray) {
        return (fieldsArray &&
            fieldsArray.length >= defaultConfigFieldsCount &&
            fieldsArray.length <= maxConfigFieldsCount);
    }
}
__decorate([
    attr({ attribute: 'chart-fields' })
], ConcreteDatasource.prototype, "chartFields", void 0);
__decorate([
    attr({ attribute: 'server-fields' })
], ConcreteDatasource.prototype, "serverFields", void 0);
__decorate([
    attr({ mode: 'boolean', attribute: 'is-stack' })
], ConcreteDatasource.prototype, "isStack", void 0);
__decorate([
    attr({ mode: 'boolean', attribute: 'is-group' })
], ConcreteDatasource.prototype, "isGroup", void 0);
__decorate([
    attr({ mode: 'boolean' })
], ConcreteDatasource.prototype, "reduced", void 0);
__decorate([
    attr({ mode: 'boolean', attribute: 'show-formatted-time' })
], ConcreteDatasource.prototype, "showFormattedTime", void 0);
let ChartDatasource = class ChartDatasource extends LifecycleMixin(ConcreteDatasource) {
    disconnectedCallback() {
        if (!this.shouldRunDisconnect)
            return;
        super.disconnectedCallback();
    }
};
ChartDatasource = __decorate([
    customElement({
        name: 'chart-datasource',
    })
], ChartDatasource);
export { ChartDatasource };
